/****************************************************************************
 *	epicsUsrConfig.c
 *
 *	Contains APS customizations for vxWorks BSPs
 *
 *	ajk - 24Feb89	
 *	joh - 27Feb89	Moved SHORTADDR, STDADDR decl to here
 *	joh - 27Feb89	Added code to save the IOC name 
 *	mrk - 03OCT97   Convert to 3.13. Place changes in separate files
 *	anj - 16Jul99	Added PPC support, epicsIncPort and bootShow
 *
 ****************************************************************************/

#include "symLib.h"
#include "sysSymTbl.h"
#include "tyLib.h"
#include "netinet/in.h"
#include "sockLib.h"

#define PROMPT_LEN 12
#define PROMPT_FMT "%.9s> "

/* __USER_LABEL_PREFIX__ is a gnu cpp macro which evaluates to 
 * "_" (unquoted) on platforms that have leading underscores
 * to their symbol names, otherwise to an empty string.
 * SYMNAME is a macro which prepends __USER_LABEL_PREFIX__ on
 * to its string argument, to generate the right symbol name. */
#define QUOTE(x) #x
#define XQUOTE(x) QUOTE(x)
#define SYMNAME(sym) XQUOTE(__USER_LABEL_PREFIX__) sym

/* This routine increments the last used local port number to remove the
 * long boot delay when talking to a Solaris ftp host.  TSR #116602 */
static void epicsIncPort(void) {
	int mybsock;
	struct sockaddr_in mybsockaddr;

	mybsockaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	mybsockaddr.sin_port = 0;
	mybsockaddr.sin_family = AF_INET;
	mybsock = socket(AF_INET, SOCK_STREAM, 0);
	bind (mybsock, (struct sockaddr *) &mybsockaddr, sizeof(mybsockaddr));
	close(mybsock);
}

/* This routine is for convenience, as different BSP architectures have
 * different values for BOOT_LINE_ADRS and we'd like to use the same
 * call independent of the architecture. */
int bootShow(void) {
	bootParamsShow(BOOT_LINE_ADRS);
	return 0;
}

static void epicsUsrInit(void)
{
	char	startupDir[BOOT_FIELD_LEN];
	char	promptStr[PROMPT_LEN];
	char	*sep;
#ifdef INCLUDE_RDB
	int	tid; /* of remote debugger task */
#endif
        void (*epicsUsrConfigInit)() = NULL;
	SYM_TYPE type;
	
        symFindByName(sysSymTbl, SYMNAME("epicsUsrConfigInit"),
		      (void *) &epicsUsrConfigInit, &type);
	if(epicsUsrConfigInit) (*epicsUsrConfigInit)();
	
	/* Use DELETE for line rubout */
	tyDeleteLineSet(0x7f);
	
	/* Find a startup directory, preferably the startup script's */
	if (strlen(sysBootParams.startupScript) > 1)
		strcpy(startupDir, sysBootParams.startupScript);
	else 
		strcpy(startupDir, sysBootFile);
	
	/* Drop everything after the last '/', '\' or ':' separator found in
	 * the directory name to get our working directory */
	sep = strrchr(startupDir, '/');
	if (!sep) sep = strrchr(startupDir, '\\');
	if (!sep) sep = strrchr(startupDir, ':');
	
	if (sep) {
		*sep = EOS;	/* Terminate string here */
		cd(startupDir);

		/* Identify the last leaf in that directory tree */
		sep = strrchr(startupDir, '/');
		if (!sep) sep = strrchr(startupDir, '\\');
		if (!sep) sep = strrchr(startupDir, ':');
		
		if (sep) {
			sprintf(promptStr, PROMPT_FMT, sep + 1);
			shellPromptSet(promptStr);
		}
	}

#ifdef INCLUDE_RDB
	/* Look for rdbTask and stop it */
	tid = taskNameToId("tRdbTask");
	if (tid != ERROR) {
		printf ("\nioc_startup: tRdbTask (vxgdb) ID=%8x \n",tid);
		taskSuspend(tid);
	}
#endif
}
